##Procedure Header
# Name:
#    ixiangpf::emulation_macsec_config
#
# Description:
#    This procedure will add and configure Macsec to a particular Ixia Interface.
#
# Synopsis:
#    ixiangpf::emulation_macsec_config
#        -mode                                     CHOICES create
#                                                  CHOICES delete
#                                                  CHOICES modify
#                                                  CHOICES enable
#                                                  CHOICES disable
#        [-port_handle                             REGEXP ^[0-9]+/[0-9]+/[0-9]+$]
#        [-handle                                  ANY]
#x       [-starting_pn                             ANY]
#x       [-starting_xpn                            ANY]
#x       [-rekey_mode                              CHOICES timerBased pnBased]
#x       [-interleave_duration                     NUMERIC]
#x       [-replay_window_size                      ANY]
#x       [-interleave_old_sak                      CHOICES 0 1]
#x       [-active_device                           ANY]
#x       [-cipher_suite                            CHOICES aes128
#x                                                 CHOICES aes256
#x                                                 CHOICES aesxpn128
#x                                                 CHOICES aesxpn256]
#x       [-confidentiality_offset                  ANY]
#x       [-enable_clear_text_vlans                 CHOICES 0 1]
#x       [-enable_encrypted_vlans                  ANY]
#x       [-encrypted_vlan_count                    NUMERIC]
#x       [-tx_sak_pool_size                        NUMERIC]
#x       [-rekey_behaviour                         CHOICES dontRekey
#x                                                 CHOICES rekeyContinuous
#x                                                 CHOICES rekeyFixedCount]
#x       [-periodic_rekey_interval                 NUMERIC]
#x       [-periodic_rekey_attempts                 NUMERIC]
#x       [-enable_confidentiality                  ANY]
#x       [-end_station                             ANY]
#x       [-include_sci                             ANY]
#x       [-system_id                               ANY]
#x       [-system_id_step                          ANY]
#x       [-port_id                                 ANY]
#x       [-rx_sak_pool_size                        NUMERIC]
#x       [-dut_sci_mac                             ANY]
#x       [-dut_sci_mac_step                        ANY]
#x       [-dut_sci_port_id                         ANY]
#x       [-dut_msb_of_xpn                          ANY]
#x       [-override_tci_setting                    ANY]
#x       [-version                                 ANY]
#x       [-tci_es_bit                              ANY]
#x       [-tci_sc_bit                              ANY]
#x       [-tci_scb_bit                             ANY]
#x       [-tci_e_bit                               ANY]
#x       [-tci_c_bit                               ANY]
#x       [-multiplier                              NUMERIC]
#x       [-name                                    ALPHA]
#x       [-tx_sak128                               ANY]
#x       [-tx_sak256                               ANY]
#x       [-tx_ssci                                 ANY]
#x       [-tx_salt                                 ANY]
#x       [-rx_sak128                               ANY]
#x       [-rx_sak256                               ANY]
#x       [-rx_ssci                                 ANY]
#x       [-rx_salt                                 ANY]
#x       [-vlan_id                                 ANY]
#x       [-priority                                ANY]
#x       [-tpid                                    CHOICES ethertype8100
#x                                                 CHOICES ethertype88a8
#x                                                 CHOICES ethertype9100
#x                                                 CHOICES ethertype9200
#x                                                 CHOICES ethertype9300]
#x       [-sec_tag_offset_unknown                  ANY]
#x       [-rx_sec_tag_offset                       ANY]
#x       [-type_of_ca                              CHOICES pairwiseca
#x                                                 CHOICES groupwithsingledut
#x                                                 CHOICES groupwithmultidut]
#x       [-max_ca_count                            CHOICES macsecmaxcaone
#x                                                 CHOICES macsecmaxcatwo
#x                                                 CHOICES macsecmaxcafour
#x                                                 CHOICES macsecmaxcaeight
#x                                                 CHOICES macsecmaxcasixteen
#x                                                 CHOICES macsecmaxcathirtytwo
#x                                                 CHOICES macsecmaxcasixtyfour
#x                                                 CHOICES macsecmaxcaonehundredtwentyeight
#x                                                 CHOICES macsecmaxcatwohundredfiftysix]
#x       [-max_dut_tx_sc_per_ca                    ANY]
#x       [-rx_sc_identifying_field                 CHOICES rxscidentifyingfieldsrcmac
#x                                                 CHOICES rxscidentifyingfieldscisystemid
#x                                                 CHOICES rxscidentifyingfieldsciportid]
#x       [-rx_sc_identifying_field_offset          ANY]
#x       [-rx_sc_identifying_field_length_in_bytes ANY]
#x       [-macsec_validation                       CHOICES check]
#x       [-replay_protection                       ANY]
#
# Arguments:
#    -mode
#        This option defines whether to Create/Modify/Delete/Enable/Disable Macsec.
#    -port_handle
#        The port on which the Macsec configuration will take place.
#    -handle
#        Specifies the parent node handle ( For ex : IP stack output of interface_config) on which Macsec is configured.
#        The Macsec handle(s) are returned by the procedure "emulation_macsec_config" when configuring Macsec on the Ixia interface.
#        For modify -mode, -handle can be Macsec's node handle.
#        For delete -mode, -handle is Macsec's node handle.
#        For enable/disable -mode, -handle can be Macsec's node handle or session(item) handle of these nodes.
#x   -starting_pn
#x       PN value with which packet transmission will begin.
#x   -starting_xpn
#x       PN value with which packet transmission will begin.
#x   -rekey_mode
#x       Rekey criteria, either PN based or Timer based.
#x   -interleave_duration
#x       Determines how long the Old and New SAKs should be interleaved.
#x   -replay_window_size
#x       Window size required for calculating LAPN.
#x   -interleave_old_sak
#x       Determines whether Ixia interleaves Old and New SAKs after Rekey is triggered.
#x   -active_device
#x       Determines whether a MACsec device is active or not. If disabled, MACsec will not be started on the device.
#x   -cipher_suite
#x       The type of cipher suite. Options include: 1) GCM-AES-1282) GCM-AES-256 3) GCM-AES-XPN-1284) GCM-AES-XPN-256
#x   -confidentiality_offset
#x       Determines the confidentiality offset for both transmit and receive channel.
#x   -enable_clear_text_vlans
#x       Encrypt the VLAN information from Ethernet to form the encrypted VLAN configuration.
#x   -enable_encrypted_vlans
#x       Enables VLANs for the sessions.
#x   -encrypted_vlan_count
#x       Determines if VLAN information to be encrypted or not. Maximum 6 VLANs can be added in the encrypted payload.
#x   -tx_sak_pool_size
#x       Determines the number of SAKs configured for each device for the Tx secure channel. Multiple SAKs are needed if Rekey scenario is to be simulated.
#x   -rekey_behaviour
#x       Determines the Rekey behavior.
#x   -periodic_rekey_interval
#x       Determines the time interval between two subsequent Rekey events. The timer starts with the first MACsec packet transmission from a device.
#x   -periodic_rekey_attempts
#x       Determines the number of times Rekey will happen after MACsec is started.
#x   -enable_confidentiality
#x       Determines whether transmitted MACsec payload is encrypted or not.
#x   -end_station
#x       Determines whether this device should act as the End Station.
#x   -include_sci
#x       Determines whether SCI should be included in the SecTAG of the transmitted MACsec traffic.
#x   -system_id
#x       System Identifier component of the SCI field with which MACsec traffic is transmitted.
#x   -system_id_step
#x       Step argument of System Id.
#x   -port_id
#x       Port ID component of SCI with which MACsec traffic is transmitted.
#x   -rx_sak_pool_size
#x       Determines the number of SAKs configured for each device for the Rx secure channel. Multiple SAKs are needed if DUT is expected to trigger Rekey during the test.
#x   -dut_sci_mac
#x       MAC component of SCI with which MACsec traffic is expected from the DUT.
#x   -dut_sci_mac_step
#x       Step argument of DUT SCI MAC.
#x   -dut_sci_port_id
#x       Port ID component of SCI with which MACsec traffic is expected from the DUT.
#x   -dut_msb_of_xpn
#x       The 32 most significant bits of the XPN that DUT will be using to construct the 64 bits XPN value when test starts.
#x   -override_tci_setting
#x       If enabled, the default TCI settings as per configuration will be allowed for overwriting. Used to simulate mal-configured SecTAG for negative testing.
#x   -version
#x       Version bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
#x   -tci_es_bit
#x       Determines the "ES" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
#x   -tci_sc_bit
#x       Determines the "SC" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
#x   -tci_scb_bit
#x       Determines the "SCB" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
#x   -tci_e_bit
#x       Determines the "E" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
#x   -tci_c_bit
#x       Determines the "C" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
#x   -multiplier
#x       Number of layer instances per parent instance (multiplier).
#x   -name
#x       Name of NGPF element, guaranteed to be unique in Scenario.
#x   -tx_sak128
#x       128 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
#x   -tx_sak256
#x       256 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
#x   -tx_ssci
#x       4 bytes Short SCI for XPN cipher suites.
#x   -tx_salt
#x       12 bytes Salt value for XPN cipher suites.
#x   -rx_sak128
#x       128 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
#x   -rx_sak256
#x       256 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
#x   -rx_ssci
#x       4 bytes Short SCI for XPN cipher suites.
#x   -rx_salt
#x       12 bytes Salt value for XPN cipher suites.
#x   -vlan_id
#x       12-bit VLAN ID in the VLAN tag.
#x   -priority
#x       3-bit user priority field in the VLAN tag.
#x   -tpid
#x       16-bit Tag Protocol Identifier (TPID) or EtherType in the VLAN tag.
#x   -sec_tag_offset_unknown
#x       Denotes that the position of the SecTAG in MACsec packets received from the DUT is unknown.
#x       It is known by default, and only in some cases DUT may insert some outer headers due to which
#x       position of SecTAG will be moved.
#x   -rx_sec_tag_offset
#x       This denotes the position of the SecTAG in macsec packets received from the DUT. The default offset is 12, but it may not be always 12 depending on different tunneling scenario.
#x   -type_of_ca
#x       Type of CA configured on the port.
#x   -max_ca_count
#x       Denotes the maximum number of CA configured on the port.
#x   -max_dut_tx_sc_per_ca
#x       Denotes the maximum DUT Tx SC that can be supported per CA.
#x   -rx_sc_identifying_field
#x       SC will identify receiving port based on the field. The options are Source MAC, SCI, SCI System ID, SCI Port ID and Custom.
#x   -rx_sc_identifying_field_offset
#x       This denotes the position of the Rx SC Identifying Field in macsec packets received from the DUT.
#x   -rx_sc_identifying_field_length_in_bytes
#x       This denotes the length in bytes of the Rx SC Identifying Field in macsec packets received from the DUT.
#x   -macsec_validation
#x       Denotes the Rx Validation behavior set at the port. Can be either Disabled, Check or Strict.
#x   -replay_protection
#x       Denotes Replay protection behavior of the port. Based on this option,
#x       out of window packets will either be discarded or passed to the upper
#x       layer control plane plugins and corresponding statistics will be incremented.
#
# Return Values:
#    A list containing the ethernet protocol stack handles that were added by the command (if any).
#x   key:ethernet_handle     value:A list containing the ethernet protocol stack handles that were added by the command (if any).
#    $::SUCCESS | $::FAILURE
#    key:status              value:$::SUCCESS | $::FAILURE
#    When status is $::FAILURE, log shows the detailed information of failure.
#    key:log                 value:When status is $::FAILURE, log shows the detailed information of failure.
#    Handle of Macsec configured
#    key:macsec_handle       value:Handle of Macsec configured
#    Handle of Tx SAK Pool configured
#    key:tx_sak_pool_handle  value:Handle of Tx SAK Pool configured
#    Handle of Rx SAK Pool configured
#    key:rx_sak_pool_handle  value:Handle of Rx SAK Pool configured
#
# Examples:
#
# Sample Input:
#
# Sample Output:
#
# Notes:
#
# See Also:
#

package ixiangpf;

use utils;
use ixiahlt;

sub emulation_macsec_config {

	my $args = shift(@_);

	my @notImplementedParams = ();
	my @mandatoryParams = ();
	my @fileParams = ();

	# ixiahlt::logHltapiCommand('emulation_macsec_config', $args);
	# ixiahlt::utrackerLog ('emulation_macsec_config', $args);

	return ixiangpf::runExecuteCommand('emulation_macsec_config', \@notImplementedParams, \@mandatoryParams, \@fileParams, $args);
}

# Return value for the package
return 1;
