# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def emulation_evpn_vxlan_wizard(self, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    emulation_evpn_vxlan_wizard
		
		 Description:
		    This procedure configures a Evpn scenario in Ixnetwork.
		
		 Synopsis:
		    emulation_evpn_vxlan_wizard
		x       [-vtep_side_port                 RANGE 1-100
		x                                        DEFAULT 1]
		x       [-host_side_port                 RANGE 0-100
		x                                        DEFAULT 1]
		x       [-enable_spine                   CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-number_of_spines_per_port      RANGE 1-100000
		x                                        DEFAULT 1]
		x       [-spine_interface_address        ALPHA]
		x       [-spine_interface_address_prefix RANGE 8-31
		x                                        DEFAULT 24]
		x       [-vtep_address                   ALPHA]
		x       [-vtep_prefix_length             RANGE 8-31
		x                                        DEFAULT 24]
		x       [-gateway_address                ALPHA]
		x       [-spine_vlan_enabled             CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-spine_vlan_id                  ALPHA]
		x       [-spine_underlaying_protocol     CHOICES ebgp ibgp ospf isis
		x                                        DEFAULT ebgp]
		x       [-spine_remote_bgp               ALPHA]
		x       [-spine_local_as                 ALPHA]
		x       [-ospf_area_id                   NUMERIC
		x                                        DEFAULT 0]
		x       [-ospf_network_type              CHOICES broadcast pointtopoint
		x                                        DEFAULT broadcast]
		x       [-isis_network_type              CHOICES broadcast pointpoint
		x                                        DEFAULT broadcast]
		x       [-isis_area_id                   ALPHA]
		x       [-isis_level                     CHOICES level1 level2 l1l2
		x                                        DEFAULT level1]
		x       [-vtep_number_per_spine          RANGE 1-100000
		x                                        DEFAULT 10]
		x       [-vtep_number_per_port           RANGE 1-100000
		x                                        DEFAULT 10]
		x       [-configure_loopback_ip          CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-vtep_loopback_address          ALPHA]
		x       [-vtep_loopback_prefix_length    RANGE 8-32
		x                                        DEFAULT 32]
		x       [-vtep_dut_loopback_address      ALPHA]
		x       [-vtep_bgp_peering_type          CHOICES ebgp ibgp
		x                                        DEFAULT ibgp]
		x       [-vtep_local_as                  ALPHA]
		x       [-vtep_multi_homing              CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-ethernet_segment_type          CHOICES type0
		x                                        CHOICES type1
		x                                        CHOICES type2
		x                                        CHOICES type3
		x                                        CHOICES type4
		x                                        CHOICES type5
		x                                        CHOICES type6
		x                                        DEFAULT type0]
		x       [-ethernet_segement_id           ALPHA]
		x       [-multi_homing_type              CHOICES singleActive allActive
		x                                        DEFAULT allActive]
		x       [-enable_irb                     CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-irb_model                      CHOICES symmetric asymmetric
		x                                        DEFAULT symmetric]
		x       [-l3vni_per_vtep                 RANGE 1-65535
		x                                        DEFAULT 1]
		x       [-service_interface_type         CHOICES vlanBasedService
		x                                        CHOICES vlanBundleService
		x                                        CHOICES vlanAwareBundle
		x                                        DEFAULT vlanBasedService]
		x       [-mac_vrf_per_vtep               NUMERIC
		x                                        DEFAULT 1]
		x       [-mac_vrfper_ip_vrf              RANGE 1-65535
		x                                        DEFAULT 1]
		x       [-bd_per_mac_vrf                 RANGE 1-10000
		x                                        DEFAULT 1]
		x       [-ethernet_tag_id_start          RANGE 0-4294967295
		x                                        DEFAULT 1]
		x       [-ethernet_tag_id_step           RANGE 0-4294967295
		x                                        DEFAULT 1]
		x       [-l2_per_l3vni                   RANGE 1-65535
		x                                        DEFAULT 1]
		x       [-l2vni_per_vtep                 RANGE 1-65535
		x                                        DEFAULT 1]
		x       [-l2vni_start                    RANGE 0-16777215
		x                                        DEFAULT 100001]
		x       [-l2vni_step                     RANGE 0-16777215
		x                                        DEFAULT 1]
		x       [-repeat_l2vni_vtep              CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-repeat_l2vni_ports             CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-l3vni_start                    RANGE 0-16777215
		x                                        DEFAULT 200001]
		x       [-l3vni_ste                      RANGE 0-16777215
		x                                        DEFAULT 1]
		x       [-repeat_l3vni_vtep              CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-repeat_l3vni_ports             CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-router_mac_address             ALPHA]
		x       [-mac_vrf_auto                   CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-macvrf_route_target_type       CHOICES as as4 ip
		x                                        DEFAULT as]
		x       [-mac_vrf_as2_byte_start         RANGE 0-65535
		x                                        DEFAULT 100]
		x       [-mac_vrf_as2_byte_step          RANGE 0-65535
		x                                        DEFAULT 0]
		x       [-mac_vrf_as4_byte_start         RANGE 0-4294967295
		x                                        DEFAULT 100]
		x       [-mac_vrf_as4_byte_step          RANGE 0-4294967295
		x                                        DEFAULT 0]
		x       [-mac_vrf_ipv4_start             IPV4
		x                                        DEFAULT 2.2.2.2]
		x       [-mac_vrf_ipv4_step              IPV4
		x                                        DEFAULT 0.0.0.0]
		x       [-mac_vrf_assigned_number_start  RANGE 0-4294967295
		x                                        DEFAULT 1]
		x       [-mac_vrf_assigned_number_step   RANGE 0-4294967295
		x                                        DEFAULT 1]
		x       [-ip_vrf_auto                    CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-ipvrf_route_target_type        CHOICES as as4 ip
		x                                        DEFAULT as]
		x       [-ip_vrf_as2_byte_start          RANGE 0-65535
		x                                        DEFAULT 100]
		x       [-ip_vrf_as2_byte_step           RANGE 0-65535
		x                                        DEFAULT 0]
		x       [-ip_vrf_as4_byte_start          RANGE 0-4294967295
		x                                        DEFAULT 100]
		x       [-ip_vrf_as4_byte_step           RANGE 0-4294967295
		x                                        DEFAULT 0]
		x       [-ip_vrf_ipv4_start              IPV4
		x                                        DEFAULT 2.2.2.2]
		x       [-ip_vrf_ipv4_step               IPV4
		x                                        DEFAULT 0.0.0.0]
		x       [-ip_vrf_assigned_number_start   RANGE 0-4294967295
		x                                        DEFAULT 1]
		x       [-ip_vrf_assigned_number_step    RANGE 0-4294967295
		x                                        DEFAULT 1]
		x       [-imet_route                     CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-multicast_tunnel_type          CHOICES tunneltypeingressreplication
		x                                        CHOICES pimsm
		x                                        CHOICES pimssm
		x                                        DEFAULT tunneltypeingressreplication]
		x       [-same_l2vni_for_multicast       CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-multicast_vni_start            RANGE 0-16777215
		x                                        DEFAULT 300001]
		x       [-multicast_vni_step             RANGE 0-16777215
		x                                        DEFAULT 1]
		x       [-repeat_vniacross_vtep          CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-repeat_vniacross_port          CHOICES 0 1
		x                                        DEFAULT 1]
		x       [-group_address                  ALPHA]
		x       [-rp_address                     IPV4
		x                                        DEFAULT 1.1.1.1]
		x       [-vm_count_per_bd                RANGE 0-100000000
		x                                        DEFAULT 1]
		x       [-vm_host_mac_address_start      MAC
		x                                        DEFAULT 22:22:00:00:00:01]
		x       [-enable_vlan                    CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-vlan_id_start                  RANGE 0-4095
		x                                        DEFAULT 1001]
		x       [-vlan_id_ste                    RANGE 0-4095
		x                                        DEFAULT 1]
		x       [-enable_mac_mobility            CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-enable_ipv4_address            CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-ipv4_address_start             IPV4
		x                                        DEFAULT 192.168.0.1]
		x       [-ipv4_subnet_mask               RANGE 0-32
		x                                        DEFAULT 24]
		x       [-enable_ipv6_address            CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-ipv6_address_start             IPV6
		x                                        DEFAULT 2000::1]
		x       [-ipv6_subnet_mask               RANGE 0-128
		x                                        DEFAULT 64]
		x       [-number_of_ip_prefix_route      RANGE 0-100000000
		x                                        DEFAULT 0]
		x       [-ipv4_prefix_start              IPV4
		x                                        DEFAULT 192.168.1.0]
		x       [-ipv4_prefix_length             RANGE 0-32
		x                                        DEFAULT 24]
		x       [-number_of_ipv6_prefix_route    RANGE 0-100000000
		x                                        DEFAULT 0]
		x       [-ipv6_prefix_start              IPV6
		x                                        DEFAULT 2222::0]
		x       [-ipv6_prefix_length             RANGE 0-128
		x                                        DEFAULT 64]
		x       [-use_case                       CHOICES interfaceLess
		x                                        CHOICES interfacefullWithCorefacingIRB
		x                                        CHOICES interfacefullWithUnnumberedCorefacingIRB
		x                                        CHOICES tsipAddressOverlayIndex
		x                                        DEFAULT interfaceLess]
		x       [-vni_for_irb                    ALPHA]
		x       [-irb_ip_address                 ALPHA]
		x       [-irb_mac                        ALPHA]
		x       [-va_ip_address                  ALPHA]
		x       [-va_ipv6_address                ALPHA]
		x       [-va_mac                         ALPHA]
		x       [-number_of_vm_host              RANGE 1-1000
		x                                        DEFAULT 1]
		x       [-host_mac_address               ALPHA]
		x       [-enable_vlan_for_vms            CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-vlan_id_for_vms                ALPHA]
		x       [-enable_ipv4_address_for_vms    CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-ipv4_address_for_vms           ALPHA]
		x       [-ipv4_subnet_mask_for_vms       RANGE 0-32
		x                                        DEFAULT 24]
		x       [-ipv4_gateway_for_vms           ALPHA]
		x       [-enable_ipv6_address_for_vms    CHOICES 0 1
		x                                        DEFAULT 0]
		x       [-ipv6_address_for_vms           ALPHA]
		x       [-ipv6_subnet_mask_for_vms       RANGE 0-128
		x                                        DEFAULT 64]
		x       [-ipv6_gateway_for_vms           ALPHA]
		
		 Arguments:
		x   -vtep_side_port
		x       Port count in Spine-Leaf topology
		x   -host_side_port
		x       Port count in VM or Host topology
		x   -enable_spine
		x       Configures Spine in topology
		x   -number_of_spines_per_port
		x       Spine count per port. This parameter is only valid if enable_spine parameter is set to True
		x   -spine_interface_address
		x       Spine Interface IP Address. This interface is connected to DUT.. This parameter is only valid if enable_spine parameter is set to True
		x   -spine_interface_address_prefix
		x       Spine Interface Subnet Mask. This interface is connected to DUT.. This parameter is only valid if enable_spine parameter is set to True
		x   -vtep_address
		x       VTEP Interface IP Address. This interface is connected to DUT.. This parameter is only valid if enable_spine parameter is set to False
		x   -vtep_prefix_length
		x       VTEP Interface Subnet Mask. This interface is connected to DUT.. This parameter is only valid if enable_spine parameter is set to False
		x   -gateway_address
		x       Interface Gateway IP Address
		x   -spine_vlan_enabled
		x       Configures VLAN in Interface
		x   -spine_vlan_id
		x       VLAN ID pattern in Interface. This parameter is only valid if spine_vlan_enabled parameter is set to True
		x   -spine_underlaying_protocol
		x       Routing protocol in Spine/VTEP
		x   -spine_remote_bgp
		x       Underlay BGP neighbour pattern. This parameter is only valid if spine_underlaying_protocol parameter is set to ebgp
		x   -spine_local_as
		x       Underlay BGP local AS Number pattern. This parameter is only valid if spine_underlaying_protocol parameter is set to ebgp
		x   -ospf_area_id
		x       Area Id of OSPF. This parameter is only valid if spine_underlaying_protocol parameter is set to ospf
		x   -ospf_network_type
		x       OSPF Network Type. This parameter is only valid if spine_underlaying_protocol parameter is set to ospf
		x   -isis_network_type
		x       IS-IS Network Type. This parameter is only valid if spine_underlaying_protocol parameter is set to isis
		x   -isis_area_id
		x       Area Id of IS-IS. This parameter is only valid if spine_underlaying_protocol parameter is set to isis
		x   -isis_level
		x       Level of IS-IS. This parameter is only valid if spine_underlaying_protocol parameter is set to isis
		x   -vtep_number_per_spine
		x       VTEP count per Spine
		x   -vtep_number_per_port
		x       VTEP count per Port
		x   -configure_loopback_ip
		x       Confiugre EVPN on loopback interface
		x   -vtep_loopback_address
		x       IP address pattern for loopback interface
		x   -vtep_loopback_prefix_length
		x       Prefix length for loopback IP address
		x   -vtep_dut_loopback_address
		x       Loopback address of DUT for BGP peering
		x   -vtep_bgp_peering_type
		x       Overlay BGP peering type
		x   -vtep_local_as
		x       Overlay BGP local AS pattern
		x   -vtep_multi_homing
		x       Configures multi-homing in VTEP
		x   -ethernet_segment_type
		x       Ethernet Segment Type. This parameter is only valid if vtep_multi_homing parameter is set to True
		x   -ethernet_segement_id
		x       Enternet Segment Id. This parameter is only valid if vtep_multi_homing parameter is set to True
		x   -multi_homing_type
		x       Type of Multi-Homing. This parameter is only valid if vtep_multi_homing parameter is set to True
		x   -enable_irb
		x       Enable Integrated Routing andBridging in VTEP
		x   -irb_model
		x       IRB model in VTEP. This parameter is only valid if enable_irb parameter is set to True
		x   -l3vni_per_vtep
		x       L3VNI count per VTEP. This parameter is only valid if enable_irb parameter is set to True
		x   -service_interface_type
		x       Service interface type in VTEP
		x   -mac_vrf_per_vtep
		x       MAC-VRF count per VTEP. This parameter is only valid if service_interface_type parameter is set to vlanAwareBundle
		x   -mac_vrfper_ip_vrf
		x       MAC-VRF count per IP-VRF
		x   -bd_per_mac_vrf
		x       Broadcast domain count per MAC-VRF. This parameter is only valid if service_interface_type parameter is set to vlanAwareBundle
		x   -ethernet_tag_id_start
		x       Etherment Tag ID in first broadcast domain of a VTEP. This parameter is only valid if service_interface_type parameter is set to vlanAwareBundle
		x   -ethernet_tag_id_step
		x       Etherment Tag ID step in broadcast domain of a VTEP. This parameter is only valid if service_interface_type parameter is set to vlanAwareBundle
		x   -l2_per_l3vni
		x       L2VNI count per L3VNI. This parameter is only valid if enable_irb parameter is set to True
		x   -l2vni_per_vtep
		x       L2VNI count per VTEP. This parameter is only valid if service_interface_type parameter is set to vlanBasedService
		x   -l2vni_start
		x       L2VNI start value in VTEP
		x   -l2vni_step
		x       L2VNI step value in VTEP
		x   -repeat_l2vni_vtep
		x       Configures same L2VNI pattern ineach VTEP
		x   -repeat_l2vni_ports
		x       Configures same L2VNI pattern inVTEPs across ports. This parameter is only valid if repeat_l2vni_vtep parameter is set to False
		x   -l3vni_start
		x       L3VNI start value in VTEP. This parameter is only valid if enable_irb parameter is set to True
		x   -l3vni_ste
		x       L3VNI step value in VTEP. This parameter is only valid if enable_irb parameter is set to True
		x   -repeat_l3vni_vtep
		x       Configures same L3VNI pattern ineach VTEP. This parameter is only valid if enable_irb parameter is set to True
		x   -repeat_l3vni_ports
		x       Configures same L3VNI pattern inVTEPs across ports. This parameter is only valid if repeat_l3vni_vtep parameter is set to False
		x   -router_mac_address
		x       Router's MAC or IRB MAC in each VTEP. This parameter is only valid if enable_irb parameter is set to True
		x   -mac_vrf_auto
		x       Configures MAC-VRF Route Target as BGP AS:L2VNI
		x   -macvrf_route_target_type
		x       Type of L2 RT used. This parameter is only valid if mac_vrf_auto parameter is set to False
		x   -mac_vrf_as2_byte_start
		x       2 byte AS Start value. This parameter is only valid if macvrf_route_target_type parameter is set to as
		x   -mac_vrf_as2_byte_step
		x       2 byte AS Step value. This parameter is only valid if macvrf_route_target_type parameter is set to as
		x   -mac_vrf_as4_byte_start
		x       4 byte AS Start value. This parameter is only valid if macvrf_route_target_type parameter is set to as4
		x   -mac_vrf_as4_byte_step
		x       4 byte AS Step value. This parameter is only valid if macvrf_route_target_type parameter is set to as4
		x   -mac_vrf_ipv4_start
		x       IPv4 format AS Start value. This parameter is only valid if macvrf_route_target_type parameter is set to ip
		x   -mac_vrf_ipv4_step
		x       IPv4 format AS Step value. This parameter is only valid if macvrf_route_target_type parameter is set to ip
		x   -mac_vrf_assigned_number_start
		x       Start value of Assigned number. This parameter is only valid if mac_vrf_auto parameter is set to False
		x   -mac_vrf_assigned_number_step
		x       Step value of Assigned number. This parameter is only valid if mac_vrf_auto parameter is set to False
		x   -ip_vrf_auto
		x       Configures IP-VRF Route Target asBGP AS:L3VNI
		x   -ipvrf_route_target_type
		x       Type of L3 RT used
		x   -ip_vrf_as2_byte_start
		x       2 byte AS Start value
		x   -ip_vrf_as2_byte_step
		x       2 byte AS Step value
		x   -ip_vrf_as4_byte_start
		x       4 byte AS Start value
		x   -ip_vrf_as4_byte_step
		x       4 byte AS Step value
		x   -ip_vrf_ipv4_start
		x       IPv4 format AS Start value
		x   -ip_vrf_ipv4_step
		x       IPv4 format AS Step value
		x   -ip_vrf_assigned_number_start
		x       Start value of Assigned number
		x   -ip_vrf_assigned_number_step
		x       Step value of Assigned number
		x   -imet_route
		x       Configures EVPN type 3 multicast route in VTEP
		x   -multicast_tunnel_type
		x       PMSI tunnel type in type 3 route. This parameter is only valid if imet_route parameter is set to True
		x   -same_l2vni_for_multicast
		x       Configures same L2VNI for unicastand multicast. This parameter is only valid if imet_route parameter is set to True
		x   -multicast_vni_start
		x       Start value of VNI for mulitcast in aVTEP
		x   -multicast_vni_step
		x       Step value of VNI for mulitcast in aVTEP
		x   -repeat_vniacross_vtep
		x       Configures same VNI pattern ineach VTEP
		x   -repeat_vniacross_port
		x       Configures same VNI pattern inVTEPs across ports
		x   -group_address
		x       Multicast group address pattern
		x   -rp_address
		x       RP address of underlay network
		x   -vm_count_per_bd
		x       Simulated VM count per BD in VTEP. This creates EVPN MAC/IP routes.
		x   -vm_host_mac_address_start
		x       Start MAC Address pattern of VMs
		x   -enable_vlan
		x       Configures VLAN in simulated VMs
		x   -vlan_id_start
		x       Start value in VLAN ID pattern ofsimulated VTEPs. This parameter is only valid if enable_vlan parameter is set to True
		x   -vlan_id_ste
		x       Step value in VLAN ID pattern ofsimulated VTEPs. This parameter is only valid if enable_vlan parameter is set to True
		x   -enable_mac_mobility
		x       Configures MAC mobility extended community inEVPN MAC/IP route
		x   -enable_ipv4_address
		x       Enables IP advertisement along with MAC inEVPN MAC/IP route
		x   -ipv4_address_start
		x       Start IPv4 Address of VMs. This parameter is only valid if enable_ipv4_address parameter is set to True
		x   -ipv4_subnet_mask
		x       IPv4 Subnet Mask of VMs. This parameter is only valid if enable_ipv4_address parameter is set to True
		x   -enable_ipv6_address
		x       Enables IPv6 advertisement along with MAC inEVPN MAC/IP route
		x   -ipv6_address_start
		x       Start IPv6 Address of VMs. This parameter is only valid if enable_ipv6_address parameter is set to True
		x   -ipv6_subnet_mask
		x       IPv6 Subnet Mask of VMs. This parameter is only valid if enable_ipv6_address parameter is set to True
		x   -number_of_ip_prefix_route
		x       EVPN IPv4 prefix route count perBD in a VTEP
		x   -ipv4_prefix_start
		x       Start value of IPv4 prefix pattern
		x   -ipv4_prefix_length
		x       Start value IPv4 prefix length
		x   -number_of_ipv6_prefix_route
		x       EVPN IPv6 prefix route count perBD in a VTEP
		x   -ipv6_prefix_start
		x       Start value of IPv6 prefix pattern
		x   -ipv6_prefix_length
		x       Start value IPv6 prefix length
		x   -use_case
		x       EVPN IP Prefix route Use case
		x   -vni_for_irb
		x       VNI pattern for SBD IRB Interface
		x   -irb_ip_address
		x       IP Address pattern of SBD IRBInterface
		x   -irb_mac
		x       MAC Address pattern of SBD IRBInterface
		x   -va_ip_address
		x       IPv4 Address pattern of VirtualAppliances
		x   -va_ipv6_address
		x       IPv6 Address pattern of VirtualAppliances
		x   -va_mac
		x       MAC Address pattern of VirtualAppliances
		x   -number_of_vm_host
		x       Number of Hosts in VM or Host topology
		x   -host_mac_address
		x       MAC Addresses in VM or Host topology
		x   -enable_vlan_for_vms
		x       Configure VLAN in VM/Hosts
		x   -vlan_id_for_vms
		x       Configure VLAN ID. This parameter is only valid if enable_vlan_for_vms parameter is set to True
		x   -enable_ipv4_address_for_vms
		x       To enable IPv4 Address in VM/Hosts
		x   -ipv4_address_for_vms
		x       IPv4 Addresses for VMs/Hosts. This parameter is only valid if enable_ipv4_address_for_vms parameter is set to True
		x   -ipv4_subnet_mask_for_vms
		x       IPv4 subnet-mask for VMs/Hosts. This parameter is only valid if enable_ipv4_address_for_vms parameter is set to True
		x   -ipv4_gateway_for_vms
		x       Gateway for VM/Hosts. This parameter is only valid if enable_ipv4_address_for_vms parameter is set to True
		x   -enable_ipv6_address_for_vms
		x       To enable IPv6 Address in VM/Hosts
		x   -ipv6_address_for_vms
		x       IPv6 Addresses for VMs/Hosts. This parameter is only valid if enable_ipv6_address_for_vms parameter is set to True
		x   -ipv6_subnet_mask_for_vms
		x       IPv6 subnet-mask for VMs/Hosts. This parameter is only valid if enable_ipv6_address_for_vms parameter is set to True
		x   -ipv6_gateway_for_vms
		x       Gateway for VM/Hosts. This parameter is only valid if enable_ipv6_address_for_vms parameter is set to True
		
		 Return Values:
		    $::SUCCESS | $::FAILURE
		x   key:status                        value:$::SUCCESS | $::FAILURE
		    On status of failure, gives detailed information.
		x   key:log                           value:On status of failure, gives detailed information.
		    A list of topology handles configured. Can be used with the topology_config Api
		x   key:topology_handle               value:A list of topology handles configured. Can be used with the topology_config Api
		    A list of devide group handles configured. Can be used with the topology_config Api
		x   key:device_group_handle           value:A list of devide group handles configured. Can be used with the topology_config Api
		    A list of network group handles configured (if any). Can be used with the network_group_config Api
		x   key:network_group_handle          value:A list of network group handles configured (if any). Can be used with the network_group_config Api
		    A list of Ipv4 prefix pool handles configured (if any). Can be used with the network_group_config Api
		x   key:ipv4_prefix_pools_handle      value:A list of Ipv4 prefix pool handles configured (if any). Can be used with the network_group_config Api
		    A list of Mac pool handles configured (if any). Can be used with the network_group_config Api
		x   key:mac_pools_handle              value:A list of Mac pool handles configured (if any). Can be used with the network_group_config Api
		    A list of Ipv4 protocol stack handles configured. Can be used with the interface_config Api
		x   key:ipv4_handle                   value:A list of Ipv4 protocol stack handles configured. Can be used with the interface_config Api
		    A list of Ipv4 loopback protocol stack handles configured. Can be used with the interface_config Api
		x   key:ipv4_loopback_handle          value:A list of Ipv4 loopback protocol stack handles configured. Can be used with the interface_config Api
		    Handle of ethernet protocol stacks configured. Can be used with the Api interface_config
		x   key:ethernet_handle               value:Handle of ethernet protocol stacks configured. Can be used with the Api interface_config
		    A list of BGP protocol stack handles configured (if any). Can be used with the emulation_bgp_config Api
		x   key:bgp_handle                    value:A list of BGP protocol stack handles configured (if any). Can be used with the emulation_bgp_config Api
		    A list of BGP IP route handles configured (if any). Can be used with the emulation_bgp_route_config Api
		x   key:ip_routes                     value:A list of BGP IP route handles configured (if any). Can be used with the emulation_bgp_route_config Api
		    A list of EVPN EVI handles configured (if any). Can be used with the emulation_bgp_route_config Api
		x   key:evpn_evi                      value:A list of EVPN EVI handles configured (if any). Can be used with the emulation_bgp_route_config Api
		    A list of Broadcast Domain handles configured (if any). Can be used with the emulation_bgp_route_config Api
		x   key:broadcast_domain_handle       value:A list of Broadcast Domain handles configured (if any). Can be used with the emulation_bgp_route_config Api
		    A list of Isis protocol handles configured (if any). Can be used with the emulation_isis_config Api
		x   key:isis_l3_handle                value:A list of Isis protocol handles configured (if any). Can be used with the emulation_isis_config Api
		    A list of Isis router handles configured (if any). Can be used with the emulation_isis_config Api
		x   key:isis_l3_router_handle         value:A list of Isis router handles configured (if any). Can be used with the emulation_isis_config Api
		    A list of Isis Route handles configured (if any). Can be used with the emulation_isis_network_group_config Api
		x   key:ipv4_prefix_interface_handle  value:A list of Isis Route handles configured (if any). Can be used with the emulation_isis_network_group_config Api
		    A list of Ospf protocol handles configured (if any). Can be used with the emulation_ospf_config Api
		x   key:ospfv2_handle                 value:A list of Ospf protocol handles configured (if any). Can be used with the emulation_ospf_config Api
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'emulation_evpn_vxlan_wizard', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
